@@ -1,5 +1,5 @@
 /* Generate ELF backend handle.
-   Copyright (C) 2000,2001,2002,2003,2004,2005,2006,2007,2008 Red Hat, Inc.
+   Copyright (C) 2000-2010 Red Hat, Inc.
    This file is part of Red Hat elfutils.
 
    Red Hat elfutils is free software; you can redistribute it and/or modify
@@ -186,7 +186,7 @@ static const char *default_core_note_type_name (uint32_t, char *buf,
 						size_t len);
 static const char *default_object_note_type_name (uint32_t, char *buf,
 						  size_t len);
-static int default_core_note (GElf_Word n_type, GElf_Word descsz,
+static int default_core_note (const GElf_Nhdr *nhdr, const char *name,
 			      GElf_Word *regs_offset, size_t *nregloc,
 			      const Ebl_Register_Location **reglocs,
 			      size_t *nitems, const Ebl_Core_Item **);
@@ -218,6 +218,7 @@ static bool default_check_object_attribute (Ebl *ebl, const char *vendor,
 					    int tag, uint64_t value,
 					    const char **tag_name,
 					    const char **value_name);
+static int default_abi_cfi (Ebl *ebl, Dwarf_CIE *abi_info);
 
 
 static void
@@ -258,6 +259,7 @@ fill_defaults (Ebl *result)
   result->syscall_abi = default_syscall_abi;
   result->check_object_attribute = default_check_object_attribute;
   result->disasm = NULL;
+  result->abi_cfi = default_abi_cfi;
   result->destr = default_destr;
   result->sysvhash_entrysize = sizeof (Elf32_Word);
 }
@@ -602,8 +604,8 @@ default_auxv_info (GElf_Xword a_type __attribute__ ((unused)),
 }
 
 static int
-default_core_note (GElf_Word n_type __attribute__ ((unused)),
-		   GElf_Word descsz __attribute__ ((unused)),
+default_core_note (const GElf_Nhdr *nhdr __attribute__ ((unused)),
+		   const char *name __attribute__ ((unused)),
 		   GElf_Word *ro __attribute__ ((unused)),
 		   size_t *nregloc  __attribute__ ((unused)),
 		   const Ebl_Register_Location **reglocs
@@ -657,6 +659,9 @@ default_debugscn_p (const char *name)
       ".debug_macinfo",
       /* DWARF 3 */
       ".debug_ranges",
+      ".debug_pubtypes",
+      /* DWARF 4 */
+      ".debug_types",
       /* SGI/MIPS DWARF 2 extensions */
       ".debug_weaknames",
       ".debug_funcnames",
@@ -746,3 +751,10 @@ default_check_object_attribute (Ebl *ebl __attribute__ ((unused)),
   *value_name = NULL;
   return false;
 }
+
+static int
+default_abi_cfi (Ebl *ebl __attribute__ ((unused)),
+		 Dwarf_CIE *abi_info __attribute__ ((unused)))
+{
+  return 0;
+}
